#!/usr/bin/python
import os
import sys
import time
from subprocess import PIPE, Popen, check_output
import argparse
import re

# Script to automatically install and test images created with WIC.
# args.core = core image type -> core-image-sato-sdk
# args.type = image type -> genericx86-64
# args.board = board to install -> minnowmax64
# args.suite = test suite to run -> auto

parser = argparse.ArgumentParser()
parser.add_argument("-t", "--type", help='image type', required=True)
parser.add_argument("-b", "--board", help='physical machine', required=True)
parser.add_argument("-c", "--core", help='core image type (core-image-sato-sdk)', required=True)
parser.add_argument("-s", "--suite", help='test suite (auto)', required=False)
parser.add_argument("-k", "--commit", help='commit hash', required=False)
parser.add_argument("-d", "--date", help='release date', required=False)
parser.add_argument("-n", "--notif", help='send notification email', default="yes", required=False)
parser.add_argument("-v", "--version", help='yocto version', default="master", required=False)
args = parser.parse_args()

################### Variables ###################

switch_ip = "192.168.99.50"
img_dir='/root/BSP_images/' # folder on switch where image files are located
wait_time = 30  # seconds to wait for image to boot on board
ab_dir = "/home/autobuilder/yocto-autobuilder/yocto-worker/nightly-wic/build/build/tmp/deploy/wic-images/"+args.type+"/mkefidisk/"+args.core+"/build/"
ab_ip = "10.237.112.85"


if args.board == "nuc":
    args.board = "NUC_1"
if not args.suite:
    args.suite = "auto"

################### Methods ###################

def print_lava(string):
    os.system('echo \"'+string+'\"')

def send_command_to_switch(command, ip=switch_ip):
    proc = Popen(args='rsh -o StrictHostKeyChecking=no root@%s "%s"' % (ip, command), stdout=PIPE, shell=True, executable='/bin/bash')
    retval = proc.communicate()[0]
    retcode = proc.returncode
    return (retcode, retval)

def run_command(command):
    proc = Popen(args=command, stdout=PIPE, shell=True, executable='/bin/bash')
    retval = proc.communicate()[0]
    retcode = proc.returncode
    return (retcode, retval)

def boot_with_stick(stick_name, board):
    command = "/root/USB_SW/SW_interface.py %s -s board -m %s -u on -p off" % (stick_name, board)
    (status, output) = send_command_to_switch(command)
    if status != 0:
        print_lava("Error switching stick!")
        exit(1)
    time.sleep(2)
    command = "/root/USB_SW/SW_interface.py %s -s board -m %s -u on -p on" % (stick_name, board)
    (status, output) = send_command_to_switch(command)
    if status != 0:
        print_lava("Error switching stick!")
        exit(1)

def minnowmax_on_stick(arch, stick_name, board):
    string = "Arch is "+str(arch)
    print_lava(string)
    boot_with_stick(stick_name, board)
    time.sleep(30)
    if (str(arch) == "32"):
      #power cycle and switch stick
      print_lava("Booting Minnowboard Max 32 bit from stick...")
      send_command_to_switch('/root/USB_SW/boot_minnow.py usb 2 32')
    else:
      #power cycle and switch stick
      print_lava("Booting Minnowboard Max 64 bit from stick...")
      send_command_to_switch("/root/USB_SW/boot_minnow.py usb 3 64")

def cherryhill_on_stick(arch, stick_name, board):
    string = "Arch is "+str(arch)
    print_lava(string)
    boot_with_stick(stick_name, board)
    time.sleep(30)
    if (str(arch) == "32"):
      #power cycle and switch stick
      print_lava("Booting Cherryhill 32 bit from stick...")
      send_command_to_switch("/root/USB_SW/boot_cherry.py install 32")
    else:
      #power cycle and switch stick
      print_lava("Booting Cherryhill 64 bit from stick...")
      send_command_to_switch("/root/USB_SW/boot_cherry.py install 64")

def nuc_on_stick(stick_name, board):
    print_lava("Booting NUC from stick...")
    boot_with_stick(stick_name, board)
    send_command_to_switch("/root/USB_SW/boot_nuc.py usb")

def check_remote_file(fisier):
    (status, output) = send_command_to_switch('ls %s' % fisier, ip=ab_ip)
    return (status, output.strip())

def wait_for_ping(host='192.168.99.200',timeout=500):
    start_time = int(time.time())
    string = "Waiting for image to come online (%s)..." % host
    print_lava(string)
    response = 1
    while response != 0:
        response = os.system("ping -c 1 "+host+"  > /dev/null 2>&1")
        if (int(start_time) + int(timeout)) < int(time.time()):
            print_lava("Timeout reached, image still not available!")
            return 1
        time.sleep(1)

    if response == 0:
      print_lava('Image is up!')
      return 0

def get_bsp_ip(board):
    dhcp_ip = ''
    command = 'ssh cornel@10.237.112.97 \"cat /etc/dhcp/dhcpd.conf\" > out'
    os.system(command)
    with open('out', 'r') as content_file:
        dhcp_conf = content_file.read()
    os.remove('out')
    dhcp_conf = dhcp_conf.split("\n")
    for i in xrange(len(dhcp_conf)):
        if (board in dhcp_conf[i]) and ("host " in dhcp_conf[i]):
           dhcp_ip = dhcp_conf[i+2].split("fixed-address")[1].strip()[:-1]
    if dhcp_ip != '':
        return dhcp_ip
    else:
        return 1

def get_image_locally(path):
    (status, fisier) = check_remote_file(os.path.join(path,"*.direct"))
    string = "Downloading image: http://%s/machines/wic-images/%s/mkefidisk/%s/build/%s" % (ab_ip, args.type, args.core, os.path.basename(fisier))
    print_lava(string)
    if (status == 0):
        if "10.237.112" in ab_ip:
            (ec, output) = send_command_to_switch("wget -q --proxy=off http://%s/machines/wic-images/%s/mkefidisk/%s/build/%s" % (ab_ip, args.type, args.core, os.path.basename(fisier)))
        else:
            (ec, output) = send_command_to_switch("wget -q http://%s/wic-images/machines/%s/mkefidisk/%s/build/%s" % (ab_ip, args.type, args.core, os.path.basename(fisier)))
        if ec != 0:
            print_lava("Failed to get image file!")
            exit(1)
        else:
            (ec, output) = send_command_to_switch("mv %s %s" % (os.path.basename(fisier).strip(), os.path.join(img_dir, os.path.basename(fisier))))
            if ec != 0:
                print_lava("Failed to move image file!")
                exit(1)
            else:
                return (0, os.path.basename(fisier).strip())
    else:
        return (1, fisier)

def booted_from_stick(ip):
    result = check_output("rsh -o StrictHostKeyChecking=no root@%s 'cat /proc/cmdline' " % ip, shell=True)
    if "root=/dev/sdb" in result:
        return True
    else:
        return False

local_conf = os.path.join(os.environ['BUILDDIR'], 'conf', 'local.conf')
local_inc = os.path.join(os.environ['BUILDDIR'], 'conf', 'local.inc')
include_line = '\n# Included by test_wic\n\ninclude %s' % os.path.basename(local_inc)

def update_local_conf(conf):
        """
        Add include_line to local.conf only if it isn't already,
        Write conf to local.inc (override)
        :param conf: Configuration to add to local.inc
        :return: OK
        """

        with open(local_conf, 'r+') as f:
            data = f.read()
            if include_line not in data:
                f.write(include_line)

        with open(local_inc, 'w') as f:
            f.write(conf)

        return 0, 'OK'

def clear_local_conf():
    """
    Delete local.inc and remove local.inc from local.conf
    :return: OK
    """

    try:
        os.remove(local_inc)
    except:
        pass

    with open(local_conf) as f:
        data_old = f.read()
        data_new = data_old.replace(include_line, '')

    with open(local_conf, 'w') as f:
        f.write(data_new)

    return 0, 'OK'

def get_log():
    (status, output) = run_command("find . | grep '%s/.*results-bitbake-worker.log'" % args.core)
    return output.strip()

################### Script ###################

(status, output) = get_image_locally(ab_dir)
if status != 0:
    print_lava("Failed to get file from autobuilder!")
    exit(1)
else:
    print_lava("Image file downloaded.")

fisier = output

# detect stick to write
print_lava("Detecting stick to write...")
(status, stick_name) = send_command_to_switch('/root/USB_SW/rezerve_stick.py %s' % args.board)

stick_name = re.sub(r'\W+', '', stick_name)
print_lava("Selected stick: "+stick_name)

# write image to stick
print_lava("Writing image on stick...")
command = '/root/USB_SW/image_on_stick.sh %s %s' % (os.path.basename(fisier), stick_name)
#print_lava(command)
(status, output) = send_command_to_switch(command)
if status != 0:
    print_lava("Error writing image to stick!")
    exit(1)
else:
    print_lava("Done writing image on stick.")


string = "Restarting %s to boot from stick" % args.board
print_lava(string)

if (args.board == "minnowmax32"):
    minnowmax_on_stick(32, stick_name, args.board)
elif (args.board == "minnowmax64") or (args.board == "minnowmax"):
    minnowmax_on_stick(64, stick_name, args.board)
elif (args.board == "cherryhill32"):
    cherryhill_on_stick(32, stick_name, args.board)
elif (args.board == "cherryhill64"):
    cherryhill_on_stick(64, stick_name, args.board)
elif ("nuc" in args.board.lower()):
    nuc_on_stick(stick_name, args.board)
else:
    boot_with_stick(stick_name, args.board)

board_ip = get_bsp_ip(args.board)

# wait to boot
string = "We presume the image is booting. Waiting %s seconds..." % str(wait_time)
print_lava(string)
time.sleep(wait_time)

if board_ip != 1:
  started = wait_for_ping(board_ip,300)
  if (started == 0):
    print_lava("Image succesfully booted.")
    time.sleep(30)
    if booted_from_stick(board_ip):
        print_lava("Wic image is booted fine from stick")
    else:
        print_lava("Wic image did not boot from stick!")
        exit(1)
  else:
    print_lava("Timeout while waiting for image to boot first time! Reverting usb.")
    command = '/root/USB_SW/SW_interface.py '+stick_name+' -s pc -m '+args.board+' -u off -p on'
    (status, output) = send_command_to_switch(command)
    exit(1)

################### Configuration ###################

conf_template = """
BB_NUMBER_THREADS ?= "16"
PARALLEL_MAKE ?= "-j 16"
MACHINE = "%s"
IMAGE_FSTYPES_append = " tar.gz"
INHERIT += "testimage"
TEST_TARGET = "simpleremote"
TEST_TARGET_IP = "%s"
TEST_SUITES = " %s"
""" % (args.type, board_ip, args.suite)

(status, output) = update_local_conf(conf_template)
if status != 0:
    print_lava("Error changing configuration file!")
    exit(1)

################### Start testing ###################

print_lava("Bitbaking required packages...")
(status, output) = run_command("bitbake rpm psplash")
if status != 0:
    clear_local_conf()
    print_lava("Error bitbaking packages: "+output)
    exit(1)

print_lava("Bitbaking package index...")
(status, output) = run_command("bitbake package-index")
if status != 0:
    clear_local_conf()
    print_lava("Error bitbaking package index: "+output)
    exit(1)

print_lava("Downloading manifest file...")
(status, output) = run_command("wget -q http://autobuilder.yoctoproject.org/pub/nightly/CURRENT/machines/%s/%s-%s.manifest" % (args.type, args.core, args.type))
if status != 0:
    (status, output) = run_command("wget -q http://autobuilder.yoctoproject.org/pub/releases/CURRENT/machines/%s/%s-%s.manifest" % (args.type, args.core, args.type))
    if status != 0:
        clear_local_conf()
        print_lava("Error downloading manifest file: "+output)
        exit(1)

where = os.path.join(os.environ['BUILDDIR'], 'tmp', 'deploy', 'images', args.type)

print_lava("Moving manifest file...")
(status, output) = run_command("mkdir -p %s " % where)
if status != 0:
    clear_local_conf()
    print_lava("Error creating manifest folder: "+output)
    exit(1)

(status, output) = run_command("mv %s-%s.manifest %s" % (args.core, args.type, where))
if status != 0:
    clear_local_conf()
    print_lava("Error moving manifest file: "+output)
    exit(1)


print_lava("Starting tests...")
run_command("bitbake %s -c testimage" % args.core)

print_lava("Tests completed on "+args.board)
clear_local_conf()

################### Update results ###################

if args.notif.lower() == "yes":
    print_lava("Converting log file to lava bundle...")
    #string = "python /lava_test/scripts/log2bundle.py %s %s %s %s %s %s %s %s " % (get_log(), 'runtime', args.version, args.board, 'weekly', args.commit, args.date, args.core)
    string = "python /lava_test/scripts/log2bundle.py %s %s %s %s %s %s %s %s " % (get_log(), 'runtime', args.version, 'wic_' + args.board, 'weekly', args.commit, args.date, args.core)
    print_lava(string)
    (status, output) = run_command(string)
    if status != 0:
        print_lava(output)
        print_lava("Could not convert log file to bundle!")
    else:
        print_lava("Results succcesfully uploaded.")


################### Cleanup ###################

print_lava("Deleting image file from usb switch...")
(status, output) = send_command_to_switch('rm %s' % os.path.join('/root', 'BSP_images', os.path.basename(fisier)))
if status != 0:
    print_lava("Could not delete it!")
else:
    print_lava("Cleanup done.")
